/*=======================================================================*
 * Program to read in a PERFECTION export file and convert it to a QUILL *
 * import format file. Suitable for Dilwyn to use !!                     *
 *                                                                       *
 * EX EXPORT_EXE;'source_file dest_file'                                 *
 *-----------------------------------------------------------------------*
 * Copyright Norman Dunbar                        31st August 1991.      *
 *-----------------------------------------------------------------------*
 * BUGS.                                                                 *
 *                                                                       *
 * None (so far)                                                         *
 *-----------------------------------------------------------------------*
 * FOIBLES.                                                              *
 *                                                                       *
 * Ignores Headers & Footers which Quill cannot import anyway.           *
 *-----------------------------------------------------------------------*
 * AMENDMENTS.                                                           *
 *                                                                       *
 * 25/06/94 - Remove 'view' abilities & tarted up display.         v1.11 *
 *            Also converted code for C68 version 4.13c                  *
 *=======================================================================*/

#include <stdio_h>
#include <stdlib_h>
#include <string_h>

/*------------------------------------------------------*
 * Define all functions that do not return an INT value *
 * otherwise the compiler generates code to adjust the  *
 * stack for INT results which can be fun !             *
 *------------------------------------------------------*/

void check_params(int, char **);
void process_files(char **);
void read_header(FILE *);
void main_loop(void);
void close_files(char **);
int  expand_tabs(int, char *);

/*-----------------------------------------------------------*
 * Set up title windows etc. Remove any default options that *
 * are not used by EXPORT to reduce final code size.         *
 *-----------------------------------------------------------*/

void consetup_title();
void (*_consetup)() = consetup_title;

char _prog_name[] = "EXPORT";
char _version[]   = "v1.11";
char _copyright[] = "Norman Dunbar 1991 & 1994";

long (*_cmdchannels)() = NULL;
long (*_stackchannels)() = NULL;
int (*_conread)() = NULL;
int (*_conwrite)() = NULL;


/*-------------------------------------*
 * When the program is EX'd the params *
 * passed to the MAIN function will be *
 * as follows :-                       *
 *                                     *
 * argc = 3                            *
 * argv[0] = EXPORT_EXE                *
 * argv[1] = source_file (Perfection)  *
 * argv[2] = dest_file   (Quill)       *
 *-------------------------------------*/

/*-----------------------------------------------*
 * Now define GLOBAL variables for all functions *
 * to access as required. This will save having  *
 * to pass them on the stack as parameters.      *
 *-----------------------------------------------*/

FILE *in_fd,     /* Perfection file id (not QDOS)  */
     *out_fd,    /* Quill file id (not QDOS)       */
     *console;   /* SCR_512x200a0x0 id (not QDOS)  */

char tabs[256];  /* All the tab stops in the file  */

int  this_tab,   /* Current position on this line  */
     page_len,   /* Number of lines per page       */


/*-----------------------------------*
 * The program starts here when EX'd *
 *-----------------------------------*/

/*=====================================================================MAIN==*/
main(int argc, char *argv[])
{
   check_params(argc, argv);     /* Validate the number of parameters  */
   process_files(argv);          /* Open all files                     */
   read_header(in_fd);           /* Read in the Perfection file header */
   main_loop();                  /* Read each character & process it   */
   close_files(argv);            /* Close all files                    */
}


/*=============================================================CHECK_PARAMS==*/
void check_params(int argc, char *argv[])
{
   /*------------------------------------------------------------*
    * Check correct parameters supplied.                         *
    *------------------------------------------------------------*/

   if (argc != 3)
   {
      printf("\n***** E R R O R ****\n\n");
      printf("USAGE : EX %s;'input_file output_file'\n\n", argv[0]);
      printf("Input_file  = Perfection export/doc filename\n");
      printf("Output_file = Quill import filename\n\n");
      exit(-15);
   }
}


/*============================================================PROCESS_FILES==*/
void process_files(char *argv[])
{
   /*-----------------------------------------*
    * Process the input, Perfection, filename *
    *-----------------------------------------*/

   printf("\nPerfection export file name : ");
   printf(argv[1]);

   if ((in_fd = fopen(argv[1], "r")) == 0)
   {
       printf("\nCannot open %s for input\n", argv[1]);
       exit(-6);
   }

   /*-------------------------------------*
    * Process the output, Quill, filename *
    *-------------------------------------*/

   printf("\nQuill import file name      : ");
   printf("%s\n", argv[2]);

   if ((out_fd = fopen(argv[2], "w")) == 0)
   {
       printf( "\nCannot open %s for output\n", argv[2]);
       fclose(in_fd);
       exit(-6);
   }
}


/*================================================================MAIN_LOOP==*/
void main_loop()
{
   /*----------------------------------------------------------*
    * The main loop, reads a character from IN_FD, converts it *
    * if required and writes it to OUT_FD.                     *
    *----------------------------------------------------------*/

   int  this_char;
   int  distance;

   this_tab = 1;
   while ((this_char = getc(in_fd)) != EOF)
   {
       switch(this_char)
       {
           case 0x09 : /* TAB */
                      distance = expand_tabs(this_tab, tabs);
                      fprintf(out_fd, "%*s", distance, "");
                      this_tab += distance;
                      break;

           case 0xCE : /* END OF PARAGRAPH */
           case 0x0A : /* END OF LINE */
                       putc(0x0A, out_fd);
                       this_tab = 1;
                       break;

           case 0xCD : /* PSEUDO SPACE */
                       putc(' ', out_fd);
                       this_tab++;
                       break;

           case 0xCC : /* HARD PAGE BREAK */
                       putc(0x0C, out_fd);
                       this_tab = 1;
                       break;

           case 0xC0 : /* UNDERLINE ON */
                       break;

           case 0xC1 : /* OFF          */
                       break;

           case 0xC2 : /* BOLD ON      */
                       break;

           case 0xC3 : /* BOLD OFF     */
                       break;

           case 0xC4 : /* SUPSCRIPT ON */
                       break;

           case 0xC5 : /* SUBSCRIPT ON */
                       break;

           case 0xC6 : /* ITALIC ON    */
                       break;

           case 0xC7 : /* LAST 3 OFF   */
                       break;

           case 0xC8 : /* STRIP 1 ON   */
                       break;

           case 0xC9 : /* STRIP 2 ON   */
                       break;

           case 0xCA : /* STRIP 3 ON   */
                       break;

           case 0xCF : /* RESET        */
                       break;

           case 0xD0 : /* STRIP 4 ON   */
                       break;

           case 0xF9 : /* REMOVE ME    */
                       break;

           default   : /* ALL THE REST */
                       putc(this_char, out_fd);
                       this_tab++;
                       break;

       }                                         /* End of SWITCH    */
   }                                             /* End While        */
}


/*==============================================================CLOSE_FILES==*/
void close_files(char *argv[])
{
   fclose(in_fd);
   fclose(out_fd);
   printf( "\n\n%s finished, %s is ready for Quill ", argv[0], argv[2]);
   printf( "to import by LINE.\n\n");
   exit(0);
}


/*==============================================================READ_HEADER==*/
void read_header(FILE *fp)
{
    char flag[4];
    static char perfect[4] = {255, 1, 67, 88}; /* Save file flag */
    int  x;
    char c;
    int  head_foot_len;

    for (x = 0; x < 4; x++) flag[x] = (char)fgetc(fp);
    if (strncmp(flag, perfect, 4)) {
       printf( "\nInput file is not a PERFECTION saved file.\n");
       fseek(fp, 0L, 0);     /* Back to start, could be EXPORT file */
       return;
    }
    head_foot_len = (256 * fgetc(fp)) + fgetc(fp);
    if (head_foot_len)
       for (x = 0; x < head_foot_len; x++) fgetc(fp);  /* Read header text */

    for (x = 0; x < 4; x++) fgetc(fp);
                                                       /* Header lines */
    head_foot_len = (256 * fgetc(fp)) + fgetc(fp);
    if (head_foot_len)
       for (x = 0; x < head_foot_len; x++) fgetc(fp);  /* Read footer text */

    for (x = 0; x < 4; x++) fgetc(fp);                 /* Footer lines */

    for (x = 0; x < 256; x++) tabs[x] = (char)fgetc(fp);  /* Tab table */
    page_len = (256 * fgetc(fp)) + fgetc(fp);          /* Page length */
}


/*==============================================================EXPAND_TABS==*/
int expand_tabs(int this_tab, char *tabs)
{
   int  x;

   for (x = this_tab + 1; x < 256; x++) {
       if (tabs[x])
           return x - this_tab + 1;
   }
   return 0;
}
